#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include "slm.h"

#define TEST "test.slm"
#define IMAGE0	"test_image/wedge.slm"
#define IMAGE1	"test_image/one.dat"
#define IMAGE2	"test_image/two.dat"
#define IMAGE3	"test_image/three.dat"
#define IMAGE4	"test_image/four.dat"
#define SLM "/dev/slm"
#define TIME	0x00000fff
#define 	QUIT		0
#define 	POWER_ON	1
#define 	POWER_OFF	2
#define 	ENABLE_WRITE	3
#define 	DISABLE_WRITE	4
#define 	UPLOAD_FRAME	5
#define 	RUN		6
#define		DISPLAY		7
#define		PERIOD		8
#define 	HELP		10

void h_err(int);

#define CHUNK (512*64)
#define ROW_SIZE (512)
#define IMAGE_LENGTH 512
#define SIZE ROW_SIZE*IMAGE_LENGTH
#define N 100000
main(){
int* dummy=NULL;
FILE	*fi,*fo;
int	dev;
unsigned char image0[SIZE];
unsigned char image1[SIZE];
unsigned char image2[SIZE];
unsigned char image3[SIZE];
unsigned char image4[SIZE];
int pos,y,i,option,offset,frame,period;
int n=0,err;
char temp[1024];

	printf("Matt's test slm driver program\n");

/* ######## IMAGE 0 ######## */
	printf("Opening test image 0\"%s\"\n",IMAGE0);
	fi=fopen(IMAGE0,"r");
	if(fi==NULL){
		printf("Image 0 %s not found\n",IMAGE0);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image0,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");

/* ######## IMAGE 1 ######## */
	printf("Opening test image 1\"%s\"\n",IMAGE1);
	fi=fopen(IMAGE1,"r");
	if(fi==NULL){
		printf("Image 1 %s not found\n",IMAGE1);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image1,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");

/* ######## IMAGE 2 ######## */
	printf("Opening test image 2\"%s\"\n",IMAGE2);
	fi=fopen(IMAGE2,"r");
	if(fi==NULL){
		printf("Image 2 %s not found\n",IMAGE2);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image2,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");

/* ######## IMAGE 3 ######## */
	printf("Opening test image 3\"%s\"\n",IMAGE3);
	fi=fopen(IMAGE3,"r");
	if(fi==NULL){
		printf("Image 3 %s not found\n",IMAGE3);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image3,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");

/* ######## IMAGE 4 ######## */
	printf("Opening test image 4\"%s\"\n",IMAGE4);
	fi=fopen(IMAGE4,"r");
	if(fi==NULL){
		printf("Image 4 %s not found\n",IMAGE4);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image4,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");

//	fo=fopen(TEST,"w");


	printf("Opening slm \"%s\"\n",SLM);
	dev=open(SLM,O_RDWR);
	if(dev<0){
		printf("Failed to open %s\n",SLM);
		perror("slm_test:FTO:");
		exit(1);
		}
	printf("Initially setting periods to %x\n",TIME);
	h_err(ioctl(dev,BNS_SET_PERIODS,TIME));
	do{
		printf("Choose an option (help=%d):\n",HELP);
		scanf("%d",&option);
		switch(option){
			case HELP:
				printf("Power on	=%d\n",POWER_ON);
				printf("Power off	=%d\n",POWER_OFF);
				printf("Enable write	=%d\n",ENABLE_WRITE);
				printf("Disable write /\n Allow image	=%d\n",DISABLE_WRITE);
				printf("Upload frames	=%d\n",UPLOAD_FRAME);
				printf("Run		=%d\n",RUN);
				printf("Display 1 frame	=%d\n",DISPLAY);
				printf("Change period	=%d\n",PERIOD);
				printf("Quit		=%d\n",QUIT);
				break;
			case POWER_ON:
				printf("Setting slm power on\n");
				h_err(ioctl(dev,BNS_POWER_ON,dummy));
				break;
			case POWER_OFF:
				printf("Setting slm power off\n");
				h_err(ioctl(dev,BNS_POWER_OFF,dummy));
				break;
			case ENABLE_WRITE:
				printf("Enabling write\n");
				h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));
				break;
			case DISABLE_WRITE:
				printf("Disabling write\n");
				h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));
				break;
			case UPLOAD_FRAME:
				/* ######## IMAGE 0 ######## */
				printf("Uploading Image 0 to frame 0");
				h_err(ioctl(dev,BNS_UPLOAD_IMAGE,0));
                                for(pos=0,y=0;y<SIZE/CHUNK;y++,pos+=CHUNK){
                                        err=write(dev,image0+pos,CHUNK);
                                        if(err!=CHUNK){
                                                printf("WRITE ERROR (writes enabled?)\n",err);
						break;
                                                }
                                        }

				/* ######## IMAGE 1 ######## */
				printf("Uploading Image 1 to frame 1");
				h_err(ioctl(dev,BNS_UPLOAD_IMAGE,1));
                                for(pos=0,y=0;y<SIZE/CHUNK;y++,pos+=CHUNK){
                                        err=write(dev,image1+pos,CHUNK);
                                        if(err!=CHUNK){
                                                printf("WRITE ERROR (writes enabled?)\n",err);
						break;
                                                }
                                        }

				/* ######## IMAGE 2 ######## */
				printf("Uploading Image 2 to frame 2");
				h_err(ioctl(dev,BNS_UPLOAD_IMAGE,2));
                                for(pos=0,y=0;y<SIZE/CHUNK;y++,pos+=CHUNK){
                                        err=write(dev,image2+pos,CHUNK);
                                        if(err!=CHUNK){
                                                printf("WRITE ERROR (writes enabled?)\n",err);
						break;
                                                }
                                        }

				/* ######## IMAGE 3 ######## */
				printf("Uploading Image 3 to frame 3");
				h_err(ioctl(dev,BNS_UPLOAD_IMAGE,3));
                                for(pos=0,y=0;y<SIZE/CHUNK;y++,pos+=CHUNK){
                                        err=write(dev,image3+pos,CHUNK);
                                        if(err!=CHUNK){
                                                printf("WRITE ERROR (writes enabled?)\n",err);
						break;
                                                }
                                        }

				/* ######## IMAGE 4 ######## */
				printf("Uploading Image 4 to frame 4");
				h_err(ioctl(dev,BNS_UPLOAD_IMAGE,4));
                                for(pos=0,y=0;y<SIZE/CHUNK;y++,pos+=CHUNK){
                                        err=write(dev,image4+pos,CHUNK);
                                        if(err!=CHUNK){
                                                printf("WRITE ERROR (writes enabled?)\n",err);
						break;
                                                }
                                        }


				printf("Done\n");
				break;

			case RUN:
				for(n=0;n<5;n++){
					h_err(ioctl(dev,BNS_DISPLAY_IMAGE,n));
					usleep(500);
				}
				break;

			case DISPLAY:
				printf("Enter a frame to display: ");
				scanf("%d",&frame);
				h_err(ioctl(dev,BNS_DISPLAY_IMAGE,frame));
				break;
			case PERIOD:
				printf("Enter period length: ");
				scanf("%x",&period);
				printf("\nSetting time to %f seconds (%d)\n",(164.4e-6*(float)period),period);
				h_err(ioctl(dev,BNS_SET_PERIODS,period));
				break;	
                        }
				
		}	
	while(option!=QUIT);
//	fclose(fo);
	printf("Setting slm power off\n");
	h_err(ioctl(dev,BNS_POWER_OFF,n));
	printf("Done, exiting\n");
	}	
	
	
void h_err(int err){
	if(err!=0){
		printf("Got error %d\n",err);
		exit(1);
		}
	else{
		printf("Ok\n");
		}
	usleep(1000);
	}
