/* $Id: manual_bns.cc,v 1.3 2004/09/30 14:52:27 ijc Exp matt $ */

/*
 * $Log: manual_bns.cc,v $
 * Revision 1.3  2004/09/30 14:52:27  ijc
 * Ian forgot to check this in what he did I do not know, Matt
 *
 * Revision 1.2  2004/08/24 11:43:33  sds
 * From this revision onwards, the "bns_main.cc" libraries are being used,
 * along with bns info file and arc info file. The old "defaults" file (which
 * stored info both about arcs and the size of the slm etc) are deprecated.
 * Added ability to change rotation direction. Also a general tidying up
 * and a bit more error checking.
 *
 */

/* manual_bns adapted from "test_slm.c" (for bns slm) and "manual_slm.cc" (for displaytech slm) */
/* sds 29th Jan 2002 */

#include "manual_bns_keys.h"
#include "../../../../hardware/bns_slm/bns_main.h"

#include <curses.h>

//#include <sys/types.h>
//#include <sys/stat.h>
//#include <fcntl.h>
//#include <sys/ioctl.h>
//#include <stdio.h>
//#include <unistd.h>
//#include <math.h>
//#include <stdlib.h>
//#include <iostream>
//#include <fstream>
using namespace std;

#define	DEFAULT_FRAME		0

// arc defaults. These should only be used if there is no arc_info_file, or the user
// specifically requests defaults
#define	DEFAULT_F		2.5e-3
#define	DEFAULT_X		0
#define	DEFAULT_W		36.3e-6
#define	DEFAULT_MSR		0.5
#define	DEFAULT_BINARY		TRUE
#define	DEFAULT_DF		100e-6
#define	DEFAULT_DX		100e-6
#define	DEFAULT_DW		0.1e-6

void	abort_and_tidy(char *,int);
void	set_defaults(struct bns_arc_characteristics *);

main(int argc,char ** argv){

// set initial values

unsigned char	image[BNS_SIZE];
struct	bns_image_characteristics bns;
struct	bns_arc_characteristics arcs;
int	n=0,ret;


float	tmpvalue;

int	quit=FALSE;
int	power_on=FALSE;
int	update_slm=FALSE;
int	first_run=TRUE;
int	altered_bns_setting=FALSE;
char	keypress;
char	kp2;
char	inptvalue[12];

	if(argc!=1){
		cout<<"No arguments!\n";
		exit(3);
		}
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	refresh();

	cout<<"Matt / Steve's manual arc manouverer for the BNS slm.\r\n\n"<<flush;
	cout<<"As of 23 August 2004:\r\n";
	cout<<"    - uses libraries common with other SLM utilities and c_scan.\r\n";
	cout<<"    - SLM image info (size, rotation, time period, laser A/B) is\r\n";
	cout<<"      stored in \""<<BNS_INFO_FILE<<"\".\r\n";
	cout<<"    - arc attributes (f, w, x, msr, df, dw, dx) are\r\n";
	cout<<"      stored in \""<<BNS_ARC_INFO_FILE<<"\".\r\n\n"<<flush;

	// initialisation...
	cout<<"Setting up SLM, loading image characteristics.\r\n"<<flush;
	ret=bns_setup(&bns,FALSE,FALSE);	// power-up=FALSE, diagnostics=FALSE
	if(ret!=BNS_OK) abort_and_tidy("bns_setup",ret);

	cout<<"\nLoading defaults...\r\n";
	first_run=bns_get_arc_characteristics(&arcs);
	if(first_run!=BNS_OK){	// no arc info file available, must be first run
		cout<<"Could not load arc info file, setting arbitrary defaults\r\n"<<flush;
		set_defaults(&arcs);
		}
	else{			// ie the values are set - pass them to the slm
		cout<<"loaded last values - sending to SLM\r\n";
		}
	ret=bns_generate_arcs(image,&bns,&arcs,FALSE);		// diag=FALSE
	if(ret!=BNS_OK) abort_and_tidy("bns_generate_arcs",ret);
	ret=bns_upload_frame(image,DEFAULT_FRAME,TRUE,FALSE);	// display_frame=TRUE, diag=FALSE
	update_slm=FALSE;
	first_run=FALSE;
	cout<<"Press `h' for help, or `q' for quit, `a' to show the arc attributes.\r\n"<<flush;

	do{
		keypress=getchar();
		switch(keypress){
			case POWER_ON:
				cout<<"Setting SLM power ON\r"<<endl;
				power_on=TRUE;
				bns_power_on();
				break;

			case POWER_OFF:
				cout<<"Setting SLM power OFF\r"<<endl;
				power_on=FALSE;
				bns_power_off();
				break;

			case LINE_SPACING_PLUS:
				arcs.w+=arcs.dw;
				cout<<"w: "<<arcs.w*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case LINE_SPACING_MINUS:
				arcs.w-=arcs.dw;
				cout<<"w: "<<arcs.w*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_PLUS:
				arcs.f+=arcs.df;
				cout<<"Mean focal length: "<<round(arcs.f*1e6)<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_MINUS:
				arcs.f-=arcs.df;
				cout<<"Mean focal length: "<<round(arcs.f*1e6)<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_PLUS:
				arcs.x+=arcs.dx;
				cout<<"x offset: "<<round(arcs.x*1e6)<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_MINUS:
				arcs.x-=arcs.dx;
				cout<<"x offset: "<<round(arcs.x*1e6)<<" microns\r"<<endl;
				update_slm=TRUE;
				break;


			case SET_TO:
				cout<<"Set "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"w.\r\nCurrent w: "<<arcs.w*1e6<<" microns. Enter new w: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.w=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"w.\r\nCurrent w: "<<arcs.w*1e6<<" microns. Enter new w: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.w=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case FOCAL_LENGTH:
						cout<<"mean focal length.\r\nCurrent mean focal length: "<<round(arcs.f*1e6)<<" microns . Enter new mean focal length: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.f=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset: "<<round(arcs.x*1e6)<<" microns. Enter new x offset: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.x=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case SLM_SIZE:
						cout<<"image size.\r\nCurrent image size: "<<round(bns.size*1e6)<<" microns. Enter new image size: "<<flush;
						echo();
						getnstr(inptvalue,10);bns.size=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						altered_bns_setting=TRUE;
						break;
					case DISTORTION_FACTOR:
						cout<<"distortion factor.\r\nCurrent distortion factor: "<<bns.distortion<<". Enter distortion factor: "<<flush;
						echo();
						getnstr(inptvalue,10);bns.distortion=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						altered_bns_setting=TRUE;
						break;
					case ROTATE:
						cout<<"rotation direction.\r\nCurrent rotation direction: "<<bns.rotate<<". Enter rotation direction (0,1,2 or 3): "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<0 or tmpvalue>3){
							cout<<"Rotation should be 0,1,2 or 3, nothing changed.\r\n"<<flush;
							}
						else{
							bns.rotate=(int)tmpvalue;
							altered_bns_setting=TRUE;
							}
						break;
					case LASER_A_GAIN:
						cout<<"laser A amplitude.\r\nCurrent laser A amplitude: "<<bns.laser_a_amp<<". Enter new laser A amplitude: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<0 || tmpvalue>255){
							cout<<"Laser A amplitude error\r\n"<<flush;
							}
						else{
							bns.laser_a_amp=(unsigned long)tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;	
					case LASER_B_GAIN:
						cout<<"laser B amplitude.\r\nCurrent laser B amplitude: "<<bns.laser_b_amp<<". Enter new laser B amplitude: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<0 || tmpvalue>255){
							cout<<"Laser B amplitude error\r\n"<<flush;
							}
						else{
							bns.laser_b_amp=(unsigned long)tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;	
					case LASER_A_TIME:
						cout<<"laser A time.\r\nCurrent laser A time: "<<bns.laser_a_time<<". Enter new laser A time: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<0 || tmpvalue>65535){
							cout<<"Laser A time error\r\n"<<flush;
							}
						else{
							bns.laser_a_time=(unsigned long)tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;	
					case LASER_B_TIME:
						cout<<"laser B time.\r\nCurrent laser B time: "<<bns.laser_b_time<<". Enter new laser B time: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<0 || tmpvalue>65535){
							cout<<"Laser B time error\r\n"<<flush;
							}
						else{
							bns.laser_b_time=(unsigned long)tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;	
					case TIME_PERIOD:
						cout<<"time period.\r\nCurrent time period: "<<bns.time_period<<" seconds. Enter new time period: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<=0 || tmpvalue>10){
							cout<<"PERIOD NOT SET. Time period must be >0 and <=10 seconds\r\n"<<endl;
							}
						else{
							bns.time_period=tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;
					case PERIOD: /* exactly same as above (TIME_PERIOD), here for human compatibility */
						cout<<"time period.\r\nCurrent time period: "<<bns.time_period<<" seconds. Enter new time period: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<=0 || tmpvalue>10){
							cout<<"PERIOD NOT SET. Time period must be >0 and <=10 seconds\r\n"<<endl;
							}
						else{
							bns.time_period=tmpvalue;
							bns_setup_timing(&bns);
							altered_bns_setting=TRUE;
							}
						break;
					case MS_RATIO:
						cout<<"mark:space ratio.\r\nCurrent mark:space ratio: "<<arcs.msr<<" Enter new mark:space ratio: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.msr=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				update_slm=TRUE;
				if(altered_bns_setting==TRUE){
					cout<<"WARNING: you have changed a value relating to the physical setup of the expt,\r\n";
					cout<<"         a value that is stored in the file \""<<BNS_INFO_FILE<<"\". Unlike the\r\n";
					cout<<"         arc attributes, this file will NOT be updated, and the change you\r\n";
					cout<<"         have just made will only affect this utility (manual_bns), and only\r\n";
					cout<<"         until it is quit.\r\n";
					cout<<"         If you wish to adjust the value permanently, you must manually edit\r\n";
					cout<<"         the file \""<<BNS_INFO_FILE<<"\". This safeguard has been put in place\r\n";
					cout<<"         to prevent you accidentally altering the image characteristics, which\r\n";
					cout<<"         would then probably mess up all your other experiments.\r\n\n"<<flush;
					altered_bns_setting=FALSE;
					}
				break;

			case FACTOR:
				cout<<"Set factor: "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"w.\r\nCurrent w factor: "<<arcs.dw*1e6<<" microns. Enter new w factor: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.dw=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"w.\r\nCurrent w factor: "<<arcs.dw*1e6<<" microns. Enter new w factor: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.dw=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case FOCAL_LENGTH:
						cout<<"focal length.\r\nCurrent mean focal length factor: "<<round(arcs.df*1e6)<<" microns. Enter new mean focal length factor: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.df=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset factor: "<<round(arcs.dx*1e6)<<" microns. Enter new x offset factor: "<<flush;
						echo();
						getnstr(inptvalue,10);arcs.dx=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				break;

			case QUIT:
				cout<<"Press 'p' to keep slm power on after quit,\n\r";
				cout<<"press 'n' to not quit,\n\r";
				cout<<"press any other key to power off slm and quit..."<<flush;
				echo();
				kp2=getchar();
				noecho();
				switch(kp2){
					case 'n':
						cout<<"Quit cancelled\r\n"<<endl;
						break;
					case 'p':
						nocbreak(); echo(); endwin();
						cout<<"Quitting, leaving SLM power on\n"<<endl;
						quit=TRUE;
						break;
					default:
						nocbreak(); echo(); endwin();
						cout<<"Quitting, turning off SLM power\n"<<endl;
						quit=TRUE;
						bns_power_off();
						break;
					}
				break;
			case ATTRIBUTES:
				cout<<"The arcs have the following characteristics:\r"<<endl;
				cout<<"Wavelength:               "<<arcs.w*1e6<<" microns\r"<<endl;
				cout<<"Mean focal length:        "<<round(arcs.f*1e6)<<" microns (to nearest micron)\r"<<endl;
				cout<<"x offset:                 "<<round(arcs.x*1e6)<<" microns (to nearest micron)\r"<<endl;
				cout<<"Mark:space ratio:         "<<arcs.msr<<"\r"<<endl<<endl;
				cout<<"Current scale factors:\r"<<endl;
				cout<<"Wavelength factor:        "<<arcs.dw*1e6<<" microns\r"<<endl;
				cout<<"Mean focal length factor: "<<round(arcs.df*1e6)<<" microns (to nearest micron)\r"<<endl;
				cout<<"x offset factor:          "<<round(arcs.dx*1e6)<<" microns (to nearest micron)\r"<<endl<<endl;
				cout<<"SLM image size:           "<<round(bns.size*1e6)<<" microns (to nearest micron)\r"<<endl;
				cout<<"Rotation direction:       "<<bns.rotate<<"\r"<<endl;
				cout<<"Distortion factor:        "<<bns.distortion<<"\r"<<endl;
				cout<<"Time period:              "<<bns.time_period<<"\r"<<endl;
				cout<<"Laser A amp:              "<<bns.laser_a_amp<<"\r"<<endl;
				cout<<"Laser A time:             "<<bns.laser_a_time<<"\r"<<endl;
				cout<<"Laser B amp:              "<<bns.laser_b_amp<<"\r"<<endl;
				cout<<"Laser B time:             "<<bns.laser_b_time<<"\r"<<endl;
				cout<<"The power is:             ";
				if(power_on==TRUE) cout<<"ON\r"<<endl<<endl; else cout<<"OFF\r"<<endl<<endl;
				break;
			case HELP:
				cout<<endl<<"Use these keys to adjust the arc attributes by the current scale factors:\r"<<endl;
				cout<<"   + -   increase/decrease w\r"<<endl;
				cout<<"   v ^   increase/decrease mean focal length\r"<<endl;
				cout<<"   < >   move x offset to the left/right\r"<<endl<<endl;
				cout<<"To set the values absolutely, press `s' followed by:\r"<<endl;
				cout<<"    w    w\r"<<endl;
				cout<<"    f    mean focal length\r"<<endl;
				cout<<"    x    x offset\r"<<endl;
				cout<<"    s    SLM size\r"<<endl;
//				cout<<"    d    distortion factor\r"<<endl;
				cout<<"    r    rotation direction\r"<<endl;
				cout<<"    m    mark:space ratio (actually the portion of light that is 'off')\r"<<endl;
				cout<<"    t    time period\r"<<endl;
				cout<<"    a    Laser A amp\r"<<endl;
				cout<<"    A    Laser A time (NOTE CAPITAL \"A\")\r"<<endl;
				cout<<"    b    Laser B amp\r"<<endl;
				cout<<"    B    Laser B time (NOTE CAPITAL \"B\")\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"To change the scale factor, press `f' followed by:\r"<<endl;
				cout<<"    w    w factor\r"<<endl;
				cout<<"    f    mean focal length factor\r"<<endl;
				cout<<"    x    x offset factor\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"    1    turn SLM power ON\r"<<endl;
				cout<<"    2    turn SLM power OFF\r"<<endl;
				cout<<"Press `q' to quit, `a' to show the arc attributes, `d' for defaults.\r"<<endl;
				break;
			case DEFAULT:
				cout<<"Image characteristics (SLM size, rotation, time period, laser A/B)\r"<<endl;
				cout<<"have been reloaded from \""<<BNS_INFO_FILE<<"\".\r\n\n";
				bns_get_image_characteristics(&bns);
				cout<<"Press: `f' to reload the arc characteristics from the file \""<<BNS_ARC_INFO_FILE<<"\",\r\n";
				cout<<"   or: `d' to set to defaults arbitrarily defined in the code of the\r\n";
				cout<<"           manual_bns program.\r"<<endl;
				echo();
				kp2=getchar();
				noecho();
				switch(kp2){
					case 'f':
						cout<<"Arc attributed loaded from \""<<BNS_ARC_INFO_FILE<<"\".\r\n"<<endl;
						bns_get_arc_characteristics(&arcs);
						break;
					case 'd':
						cout<<"Arc attributes set from defaults defined within manual_bns code.\r\n"<<endl;
						set_defaults(&arcs);
						break;
					default:
						cout<<"No defaults set.\r\n"<<endl;
						break;
					}
				update_slm=TRUE;
				break;
			case RETURN:
				cout<<"\r"<<endl;
				break;
			default:
				break;
			}

		if(update_slm==TRUE){
			ret=bns_generate_arcs(image,&bns,&arcs,FALSE);		// diag=FALSE
			if(ret!=BNS_OK) abort_and_tidy("bns_generate_arcs",ret);
			ret=bns_upload_frame(image,DEFAULT_FRAME,TRUE,FALSE);	// display_frame=TRUE, diag=FALSE
			if(ret!=BNS_OK) abort_and_tidy("bns_upload_frame",ret);
			update_slm=FALSE;
			first_run=FALSE;
			}		
		}while(quit!=TRUE);

	cout<<"Saving arc attributes... note that changes to the image attributes\n";
	cout<<"(slm size, rotation, time period, laser A/B) have NOT been saved.\n";
	bns_save_arc_characteristics(&arcs);
	}


void	abort_and_tidy(char * msg,int ret){
	refresh();nocbreak(); echo(); endwin();
	printf("Oh dear, something's gone wrong! I got a return value of %d. Quitting...\n",ret);
	printf("%s\n",msg);
	switch(ret){
		case 11: 
			printf("Probably a missing info file\n");
			break;
		case 2:
			printf("Probably manual_bns is already open somewhere else\n");
			break;
		default:
			printf("Unkown error - ask Steve! \n");
		}
	exit(ret);
	}

void	set_defaults(struct bns_arc_characteristics *arcs){
	arcs->f=DEFAULT_F;
	arcs->x=DEFAULT_X;
	arcs->w=DEFAULT_W;
	arcs->msr=DEFAULT_MSR;
	arcs->binary=DEFAULT_BINARY;
	arcs->df=DEFAULT_DF;
	arcs->dx=DEFAULT_DX;
	arcs->dw=DEFAULT_DW;
	}
