/* $Id: set_bns.cc,v 1.2 2004/08/24 14:53:16 sds Exp $ */

/*
 * $Log: set_bns.cc,v $
 * Revision 1.2  2004/08/24 14:53:16  sds
 * Quite an extensive re-write. Now uses the "bns_main" libraries,
 * and grabs the slm image info from the BNS_INFO_FILE.
 * In terms of usage, the following has changed:
 * (1) turning on the power is no longer the default. By default
 *     the power is NOT turned on. To turn on the power, use
 *     the "-p" switch. (the -no_power switch is deprecated but
 *     still allowed)
 * (2) -rotate argument added
 * (3) -msr argument added
 * (4) -greyscale argument added
 * (5) -frame argument added
 * (6) -help argument added
 * (7) -l and -f are now REQUIRED ARGUMENTS
 *
 */

/* This is set_bns, used for applying arcs to the BNS SLM from the command line.
 * Original version sds 11th Feb 2002
 * As of version 1.2, uses "bns_main" libraries and BNS_INFO_FILE and
 * BNS_ARC_INFO_FILE to keep track of arc/image characteristics. */

#include "../../../../hardware/bns_slm/bns_main.h"

/* This utility insists on receiving values for f, and w. The other
 * characteristics (msr and binary/greyscale) are set here as #defines.
 * The program does not access the BNS_ARC_INFO_FILE, although it of
 * course uses the information contained in the BNS_INFO_FILE about
 * the physical setup. */
#define	DEFAULT_MSR	0.5
#define	DEFAULT_BINARY	TRUE
#define	DEFAULT_X	0
#define	DEFAULT_FRAME	0

BOOL	sc(char *,char *);
void	print_help(struct bns_image_characteristics *, int);

int	main(int argc,char ** argv){
unsigned char	image[BNS_SIZE];
struct	bns_image_characteristics bns;
struct	bns_arc_characteristics arcs;

int	ret,index=1;
float	tmpvalue;
int	frame;
BOOL	power_on	= FALSE;
BOOL	verbose_mode	= FALSE;
BOOL	got_f		= FALSE;
BOOL	got_w		= FALSE;

	ret=bns_get_image_characteristics(&bns); if(ret!=BNS_OK) exit(ret);

	arcs.x		= DEFAULT_X;
	arcs.msr	= DEFAULT_MSR;
	arcs.binary	= DEFAULT_BINARY;
	frame		= DEFAULT_FRAME;

	if(argc==1){
		cout<<"Need some arguments!"<<endl<<endl;
		print_help(&bns,frame);
		exit(1);
		}

	while(index<argc){
		if(sc(argv[index],"-h")||sc(argv[index],"-help")||sc(argv[index],"-?")){
			print_help(&bns,frame);
			exit(1);
			}
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-n")||sc(argv[index],"-no_power")){
			power_on=FALSE;
			}
		if(sc(argv[index],"-p")||sc(argv[index],"-power")){
			power_on=TRUE;
			}
		if(sc(argv[index],"-x")||sc(argv[index],"-xoff")){
			sscanf(argv[++index],"%f",&arcs.x);
			}
		if(sc(argv[index],"-f")||sc(argv[index],"-focal_length")){
			sscanf(argv[++index],"%f",&arcs.f);
			got_f=TRUE;
			}
 		if(sc(argv[index],"-l")||sc(argv[index],"-wavelength")){
			sscanf(argv[++index],"%f",&arcs.w);
			got_w=TRUE;
			}
		if(sc(argv[index],"-m")||sc(argv[index],"-msr")){
			sscanf(argv[++index],"%f",&arcs.msr);
			}
		if(sc(argv[index],"-g")||sc(argv[index],"-greyscale")){
			arcs.binary=FALSE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&bns.time_period);
			}
		if(sc(argv[index],"-s")||sc(argv[index],"-slm_height")){
			sscanf(argv[++index],"%f",&bns.size);
			}
		if(sc(argv[index],"-r")||sc(argv[index],"-rotate")){
			sscanf(argv[++index],"%d",&bns.rotate);
			}
		if(sc(argv[index],"-d")||sc(argv[index],"-distortion")){
			sscanf(argv[++index],"%f",&bns.distortion);
			}
		if(sc(argv[index],"-fr")||sc(argv[index],"-frame")){
			sscanf(argv[++index],"%d",&frame);
			}
		index++;
		}

	if(got_f==FALSE || got_w==FALSE){
		cout<<"Error: you need to supply, at the very LEAST, f and w."<<endl<<endl;
		print_help(&bns,frame);
		exit(2);
		}

	if(verbose_mode==TRUE){
		cout<<"Setting SLM arc attributes:\n"<<flush;
		cout<<"Wavelength:               "<<arcs.w*1e6<<" microns"<<endl;
		cout<<"Mean focal length:        "<<arcs.f*1e6<<" microns"<<endl;
		cout<<"x offset:                 "<<arcs.x*1e6<<" microns"<<endl;
		if(arcs.binary==TRUE){
			cout<<"Binary distribution, msr: "<<arcs.msr<<endl<<endl;
			}
		else{
			cout<<"Greyscale distribution (msr ignored)"<<endl<<endl;
			}
		cout<<"SLM image height:         "<<bns.size*1e6<<" microns"<<endl;
		cout<<"Rotation direction:       "<<bns.rotate<<endl;
		cout<<"Distortion factor:        "<<bns.distortion<<endl;
		cout<<"Time period:              "<<bns.time_period<<endl;
		cout<<"Frame:                    "<<frame<<endl;
		cout<<"The power is ";
		if(power_on==TRUE){
			cout<<"being set:   ON"<<endl;
			}
		else cout<<"NOT being set"<<endl;
		}

	/* Note we don't use the bns_setup() function because this calls the
	 * bns_get_image_characteristics() fn, which will wipe out all the values
	 * entered on the command line. The following 4 commands are normally done
	 * as a block using bns_setup()*/

	/* (1) Open the device */
	ret=bns_open(); if(ret!=BNS_OK) exit(ret);

	/* (2) Set up the timing and laser A/laser B */
	ret=bns_setup_timing(&bns); if(ret!=BNS_OK) exit(ret);

	/* (3) Turn on power (if required) */
	if(power_on==TRUE){
		ret=bns_power_on(); if(ret!=BNS_OK) exit(ret);
		}

	/* (4) Disable writes (set DPEN=1) */
	bns_disable_writes();

	/* Generate the arcs */
	ret=bns_generate_arcs(image,&bns,&arcs,FALSE); if(ret!=BNS_OK) exit(ret); // diag=FALSE
	/* Upload the arc image to a frame and display it */
	ret=bns_upload_frame(image,frame,TRUE,FALSE);   // display_frame=TRUE, diag=FALSE

	return 0;
	}

BOOL    sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
	}

void	print_help(struct bns_image_characteristics *bns, int frame){
	cout<<"\nHelp information for \"set_bns\"\n";
	cout<<"set_bns uploads arcs to the BNS SLM. The physical characteristics of\n";
	cout<<"the SLM (size, rotation, time period etc) are defined in the file\n";
	cout<<"\""<<BNS_INFO_FILE<<"\", whilst the [non-required] arc characteristics\n";
	cout<<"(x offset, msr, binary or greyscale) are defined in the set_bns source code.\n\n";

	cout<<"REQUIRED ARGUMENTS:\n";
	cout<<"-f    -focal_length    : focal length (in metres)  REQUIRED ARGUMENT\n";
	cout<<"-l    -wavelength      : line spacing (in metres)  REQUIRED ARGUMENT\n\n";

	cout<<"OPTIONAL ARGUMENTS (defaults defined in \""<<BNS_INFO_FILE<<"\"):\n";
	cout<<"-t    -time_period     : time period (in seconds)  default "<<bns->time_period<<endl;
	cout<<"-s    -slm_height      : slm height (in metres)    default "<<bns->size<<endl;
	cout<<"-r    -rotate          : rotation direction        default "<<bns->rotate<<endl;
	cout<<"-d    -distortion      : distortion factor         default "<<bns->distortion<<endl<<endl;
	cout<<"OPTIONAL ARGUMENTS (defaults defined in \"set_bns.cc\" source code):\n";
	cout<<"-fr   -frame           : frame for arc upload      default "<<frame<<endl;
	cout<<"-x    -xoff            : x offset (in metres)      default "<<DEFAULT_X<<endl;
	cout<<"-m    -msr             : mark:space ratio (\% \"on\") default "<<DEFAULT_MSR<<endl;
	cout<<"-g    -greyscale       : greyscale, not binary     default ";if(DEFAULT_BINARY==TRUE)cout<<"BINARY\n"; else cout<<"GREYSCALE\n";
	cout<<"-n    -no_power        : do NOT turn on SLM power  (default)"<<endl;
	cout<<"-p    -power           : DO turn on SLM power      (not default)"<<endl;
	cout<<"-v    -verbose         : confirm all settings      (not default)"<<endl;
	cout<<"-h,-? -help            : print this help information"<<endl;
	}

