/* $Id: upload1_bns.c,v 1.2 2004/08/24 15:57:43 sds Exp $ */

/*
 * $Log: upload1_bns.c,v $
 * Revision 1.2  2004/08/24 15:57:43  sds
 * A bit of a re-write, now uses the "bns_main" libraries, and
 * uses the BNS_INFO_FILE to find out the default time period.
 * In terms of usage, the only thing that's changed is that by
 * default, the SLM power is NOT turned on (power was turned on
 * by default in the old version, and required a "-n" switch
 * to not turn the power on). Power can now be turned on by
 * using the "-p" switch.
 *
 */

/* This is upload1_bns, a simple command line utility for loading
 * an image from disk, uploading to a frame on the BNS SLM buffer,
 * and displaying it. */

#include "../../../../hardware/bns_slm/bns_main.h"

#define	DEFAULT_FRAME	0

BOOL sc(char *,char *);
void	print_help(struct bns_image_characteristics *, int);

main(int argc,char ** argv){

unsigned char	image[BNS_SIZE];
struct	bns_image_characteristics bns;
int	ret, index=1;
char	*filename;
BOOL	got_filename=FALSE;
float	period;
BOOL	set_period=FALSE;
int	frame=0;
BOOL	power_on=FALSE;
BOOL	verbose_mode=FALSE;
FILE	*fi;

	ret=bns_get_image_characteristics(&bns); if(ret!=BNS_OK) exit(ret);

	if(argc==1){
		printf("Need some arguments!\n\n");
		print_help(&bns,frame);
		exit(1);
		}

	while(index<argc){
                if(sc(argv[index],"-h")||sc(argv[index],"-help")||sc(argv[index],"-?")){
                        print_help(&bns,frame);
                        exit(1);
                        }
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-n")||sc(argv[index],"-no_power")){
			power_on=FALSE;
			}
		if(sc(argv[index],"-p")||sc(argv[index],"-power")){
			power_on=TRUE;
			}
		if(sc(argv[index],"-f")||sc(argv[index],"-filename")){
                        filename=argv[++index];
			got_filename=TRUE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&period);
			set_period=TRUE;
			}
		if(sc(argv[index],"-fr")||sc(argv[index],"-frame")){
			sscanf(argv[++index],"%d",&frame);
			}
		index++;
		}

	if(got_filename==FALSE){
		printf("ERROR: you must provide a filename!\n\n");
		print_help(&bns,frame);
		exit(1);
		}

	ret=bns_setup(&bns,power_on,FALSE); if(ret!=BNS_OK) exit(ret);	// diagnostics=FALSE

	if(verbose_mode==TRUE){
		printf("Uploading \"%s\" to frame %d, power has ",filename,frame);
        	if(power_on==FALSE) printf("NOT");
		printf(" been turned on");
		}

	if(set_period==TRUE){
		if(verbose_mode==TRUE){
			printf(", period: %fs",period);
			}
		bns.time_period=period;
		bns_setup_timing(&bns);
		}

	if(verbose_mode==TRUE){
		printf("\n");
		}

	ret=bns_load_and_upload_frame(filename, frame, TRUE, FALSE); if(ret!=BNS_OK) exit(ret); // display_frame=TRUE, diag=FALSE

	return 0;
	}	

BOOL	sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
        }

void	print_help(struct bns_image_characteristics *bns, int frame){
	printf("Help information for \"upload1_bns\"\n");
	printf("upload1_bns is a simple utility for loading an image from disk\n");
	printf("(it should be a 512x512 array of floats, and will probably have a\n");
	printf("\".bns\" extension), upload this image to a frame of your choice\n");
	printf("on the BNS SLM buffer, and then display the frame. The only\n");
	printf("SLM physical characteristic used by this program is the time\n");
	printf("period; rotation is NOT used, this only has relevance to arcs.\n\n");

	printf("REQUIRED ARGUMENTS:\n");
	printf("-f   -filename    : filename of image (should have .bns extension)\n\n");
	printf("OPTIONAL ARGUMENTS:\n");
	printf("-fr  -frame       : frame to upload image to  default %d\n",frame);
	printf("-t   -time_period : time period (in seconds)  default %f\n",bns->time_period);
	printf("-n   -no_power    : do NOT turn on SLM power  (not default)\n");
	printf("-p   -power       : DO turn on SLM power      (default)\n");
	printf("-v   -verbose     : confirm what's been done  (not default)\n");
	}
